@@ -27,14 +27,16 @@
 #
 # =================================================================
 
-__version__ = '0.8.0'
+__version__ = '0.10.0'
 
 import click
 from pygeoapi.openapi import generate_openapi_document
 
 
-cli = click.Group()
-cli.version = __version__
+@click.group()
+@click.version_option(version=__version__)
+def cli():
+    pass
 
 
 @cli.command()
