@@ -2,7 +2,7 @@
 #
 # Authors: Tom Kralidis <tomkralidis@gmail.com>
 #
-# Copyright (c) 2020 Tom Kralidis
+# Copyright (c) 2021 Tom Kralidis
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation
@@ -30,7 +30,6 @@
 
 import importlib
 import logging
-import os
 
 LOGGER = logging.getLogger(__name__)
 
@@ -40,18 +39,28 @@ PLUGINS = {
     'provider': {
         'CSV': 'pygeoapi.provider.csv_.CSVProvider',
         'Elasticsearch': 'pygeoapi.provider.elasticsearch_.ElasticsearchProvider',  # noqa
+        'ElasticsearchCatalogue': 'pygeoapi.provider.elasticsearch_.ElasticsearchCatalogueProvider',  # noqa
         'GeoJSON': 'pygeoapi.provider.geojson.GeoJSONProvider',
         'OGR': 'pygeoapi.provider.ogr.OGRProvider',
         'PostgreSQL': 'pygeoapi.provider.postgresql.PostgreSQLProvider',
         'SQLiteGPKG': 'pygeoapi.provider.sqlite.SQLiteGPKGProvider',
         'MongoDB': 'pygeoapi.provider.mongo.MongoProvider',
-        'FileSystem': 'pygeoapi.provider.filesystem.FileSystemProvider'
+        'FileSystem': 'pygeoapi.provider.filesystem.FileSystemProvider',
+        'rasterio': 'pygeoapi.provider.rasterio_.RasterioProvider',
+        'xarray': 'pygeoapi.provider.xarray_.XarrayProvider',
+        'MVT': 'pygeoapi.provider.mvt.MVTProvider',
+        'TinyDBCatalogue': 'pygeoapi.provider.tinydb_.TinyDBCatalogueProvider',
+        'xarray-edr': 'pygeoapi.provider.xarray_edr.XarrayEDRProvider'
     },
     'formatter': {
         'CSV': 'pygeoapi.formatter.csv_.CSVFormatter'
     },
     'process': {
         'HelloWorld': 'pygeoapi.process.hello_world.HelloWorldProcessor'
+    },
+    'process_manager': {
+        'Dummy': 'pygeoapi.process.manager.dummy.DummyManager',
+        'TinyDB': 'pygeoapi.process.manager.tinydb_.TinyDBManager'
     }
 }
 
@@ -68,10 +77,6 @@ def load_plugin(plugin_type, plugin_def):
 
     name = plugin_def['name']
 
-    if name == "OGR":
-        os.environ["OGR_GEOJSON_MAX_OBJ_SIZE"] = os.environ.get(
-            "OGR_GEOJSON_MAX_OBJ_SIZE", "20MB")
-
     if plugin_type not in PLUGINS.keys():
         msg = 'Plugin type {} not found'.format(plugin_type)
         LOGGER.exception(msg)
@@ -97,6 +102,7 @@ def load_plugin(plugin_type, plugin_def):
     module = importlib.import_module(packagename)
     class_ = getattr(module, classname)
     plugin = class_(plugin_def)
+
     return plugin
 
 
