@@ -38,16 +38,19 @@ class BaseProcessor:
     def __init__(self, processor_def, process_metadata):
         """
         Initialize object
-        :param processor_def: processor definition
-        :returns: pygeoapi.processors.base.BaseProvider
-        """
 
+        :param processor_def: processor definition
+        :param process_metadata: process metadata `dict`
+
+        :returns: pygeoapi.processor.base.BaseProvider
+        """
         self.name = processor_def['name']
         self.metadata = process_metadata
 
     def execute(self):
         """
         execute the process
+
         :returns: dict of process response
         """
 
@@ -57,6 +60,11 @@ class BaseProcessor:
         return '<BaseProcessor> {}'.format(self.name)
 
 
-class ProcessorExecuteError(Exception):
+class ProcessorGenericError(Exception):
+    """processor generic error"""
+    pass
+
+
+class ProcessorExecuteError(ProcessorGenericError):
     """query / backend error"""
     pass
